use http::handling::{
    methods::Method, 
    routes::{Route, Data},
    request::Request,
    response::{Response, Outcome, Status}, 
    file_handlers::NamedFile};

fn static_files_handler(request: Request, _data: Data) -> Outcome<Response, Status, Data> {
    println!("{:?} {}", request.uri, request.uri);
    let response = static_files(&request.uri.to_string());
    let response = match response {
        Ok(dat) => Response {
            headers: vec![],
            cookies: None,
            status: Some(Status::Ok),
            body: Box::new(dat),
        },
        Err(_) => return Outcome::Failure(Status::NotFound),
    };
    Outcome::Success(response)
}

fn static_files(path: &str) -> Result<NamedFile, Status> {
    NamedFile::open(("static/".to_string() + path).into())
}

fn index_handler(_request: Request, _data: Data) -> Outcome<Response, Status, Data> {
    Outcome::Success(Response { headers: vec![], cookies: None, status: None, body: Box::new(index()) })
}

fn index() -> NamedFile {
    NamedFile::open("templates/index.html".into()).unwrap()
}

fn favicon_handler(_request: Request, _data: Data) -> Outcome<Response, Status, Data> {
    let response = Response {
        headers: vec![],
        cookies: None, 
        status: None,
        body: Box::new(NamedFile::open("/assets/favicon.svg".into()).unwrap())
    };
    Outcome::Success(response)
}

#[tokio::main]
async fn main() {
    let index_route = Route {
        format: None,
        handler: index_handler,
        name: Some("index"),
        uri: "".try_into().unwrap(),
        method: Method::Get,
        rank: 0,
    };

    let static_route = Route { 
        format: None,
        handler: static_files_handler,
        name: Some("static files"),
        uri: "<file..>".try_into().unwrap(),
        method: Method::Get,
        rank: 0
    };

    let favicon = Route {
        format: None, 
        handler: favicon_handler,
        name: Some("favicon"),
        uri: "favicon.ico".try_into().unwrap(),
        method: Method::Get,
        rank: 0
    };

    // http::build("127.0.0.1:8000")
    http::build("127.0.0.1:8443", "127.0.0.1:8080")
        .await
        .mount("/".try_into().unwrap(), vec![index_route, favicon])
        .mount("/static".try_into().unwrap(), vec![static_route])
        .launch()
        .await;
}
